@@ -4,7 +4,7 @@
 #include <xview/svrimage.h>
 #include "workman_ui.h"
 
-static char *ident = "@(#)setup.c	1.19 11/16/93";
+static char *ident = "@(#)setup.c	1.23 1/30/94";
 
 /*
  * Define a few macros to make relative positioning easier.
@@ -658,7 +658,7 @@ about_objects_init(ip, owner)
 
 	ip->message1 = xv_create(ip->controls3, PANEL_MESSAGE,
 		XV_Y, 8,
-		PANEL_LABEL_STRING, "WorkMan 1.1",
+		PANEL_LABEL_STRING, "WorkMan 1.2",
 		PANEL_LABEL_BOLD, TRUE, NULL);
 	ip->message2 = xv_create(ip->controls3, PANEL_MESSAGE,
 		PANEL_LABEL_STRING, "\"The Civilized CD Player\"",
@@ -701,6 +701,11 @@ about_objects_init(ip, owner)
 		PANEL_LABEL_STRING, "notified of future revisions!",
 		PANEL_LABEL_BOLD, FALSE, NULL);
 
+	ip->drive = xv_create(ip->controls3, PANEL_MESSAGE,
+		PANEL_LABEL_STRING, "MMMMMMMM MMMMMMMMM",
+		PANEL_LABEL_BOLD, FALSE,
+		NULL);
+
 	space = 0;
 	PANEL_EACH_ITEM(ip->controls3, item)
 		xv_set(item, XV_KEY_DATA, INSTANCE, ip, NULL);
@@ -708,6 +713,8 @@ about_objects_init(ip, owner)
 			space = wid(item);
 	PANEL_END_EACH;
 
+	xv_set(ip->drive, PANEL_LABEL_STRING, "Unknown drive type", NULL);
+
 	space += 10;
 	xv_set(ip->controls3, XV_WIDTH, space, NULL);
 	center_x(ip->message1, ip->controls3);
@@ -716,6 +723,7 @@ about_objects_init(ip, owner)
 	center_x(ip->message4, ip->controls3);
 	center_x(ip->message5, ip->controls3);
 	center_x(ip->message6, ip->controls3);
+	center_x(ip->drive, ip->controls3);
 	xv_set(ip->message7, XV_X, space / 2 - wid(ip->message7) - 5, NULL);
 
 	put_down(ip->message2, ip->message1, basic_spacing);
@@ -725,13 +733,29 @@ about_objects_init(ip, owner)
 	put_down(ip->message4, ip->message3, basic_spacing);
 	put_down(ip->message5, ip->message4, basic_spacing * 2);
 	put_down(ip->message6, ip->message5, basic_spacing);
+	put_down(ip->drive, ip->message6, basic_spacing * 2);
 
-	xv_set(ip->controls3, XV_HEIGHT, bot(ip->message6) + 5, NULL);
+	xv_set(ip->controls3, XV_HEIGHT, bot(ip->drive) + 5, NULL);
 	window_fit(ip->about);
 
 	return (ip);
 }
 
+/*
+ * We've discovered the drive type, so update the About popup.
+ */
+void
+about_set_drivetype(vendor, model, rev)
+	char	*vendor, *model, *rev;
+{
+	extern about_objects    *Workman_about;
+	char			drivetype[21];
+
+	sprintf(drivetype, "%s %s", vendor, model);
+	xv_set(Workman_about->drive, PANEL_LABEL_STRING, drivetype, NULL);
+	center_x(Workman_about->drive, Workman_about->controls3);
+}
+
 /*
  * Create the Goodies window and its contents.
  */
