@@ -1,10 +1,10 @@
 /*
- * @(#)workman_stubs.c	1.126	11/17/93
+ * @(#)workman_stubs.c	1.133	1/24/94
  *
  * workman_stubs.c - Notify and event callback function stubs.
  */
 
-static char *ident = "@(#)workman_stubs.c	1.126\t11/17/93";
+static char *ident = "@(#)workman_stubs.c	1.133\t1/24/94";
 
 #include <stdio.h>
 #include <sys/param.h>
@@ -75,7 +75,8 @@ goodies_objects	*Workman_goodies;
 plpopup_objects	*Workman_plpopup;
 
 extern int cur_track, cur_pos_abs, cur_pos_rel, cur_tracklen, cur_cdlen,
-	cur_cdmode, cur_ntracks, cur_lasttrack, cur_firsttrack, cur_listno;
+	cur_ntracks, cur_lasttrack, cur_firsttrack, cur_listno;
+extern enum cd_modes cur_cdmode;
 extern int cur_frame;
 extern int cd_fd;
 extern int exit_on_eject, suppress_locking;
@@ -100,7 +101,7 @@ int window_is_open;
 int was_repeating = 0;
 int info_modified = 0;
 int show_titles = 1;
-#ifdef hpux
+#if defined(hpux) || defined(__bsdi__)
 int dismiss_button = 1;
 #else
 int dismiss_button = 0;
@@ -108,6 +109,11 @@ int dismiss_button = 0;
 
 Attr_attribute	INSTANCE;
 
+#ifdef __sony_news
+/* XXX -- anyone know why XView messes up on NEWS without this? */
+static int error_handler(void *d, void *e) { return (0); }
+#endif
+
 main(argc, argv)
 	int	argc;
 	char	**argv;
@@ -117,11 +123,22 @@ main(argc, argv)
 	char		*name;
         extern char     *optarg, *rcfile, *dbfiles;
 	extern int	keep_open;
+
+#ifdef SVR4
+        if (setreuid(-1, getuid()) < 0)
+        {
+                perror("setreuid");
+                exit(-1);
+        }
+#endif
         
 	/*
 	 * Initialize XView.
 	 */
 	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);
+#ifdef __sony_news
+	XSetErrorHandler(&error_handler);
+#endif
 	INSTANCE = xv_unique_key();
 
 	/*
@@ -217,10 +234,6 @@ main(argc, argv)
 			exit(1);
 		}
 
-	/* Find the CD device. */
-	if (! cd_device)
-		find_cdrom();
-
 	/* Cook the database filenames, or construct them if none were given. */
 	split_workmandb();
 
@@ -291,8 +304,8 @@ main(argc, argv)
 			NULL);
 		xv_set(button, XV_X, ((int) xv_get(Workman_about->about,
 			XV_WIDTH) - (int) xv_get(button, XV_WIDTH)) / 2,
-			XV_Y, (int) xv_get(Workman_about->message6, XV_Y) +
-			(int) xv_get(Workman_about->message6, XV_HEIGHT) + 5,
+			XV_Y, (int) xv_get(Workman_about->drive, XV_Y) +
+			(int) xv_get(Workman_about->drive, XV_HEIGHT) + 5,
 			NULL);
 		button = xv_create(Workman_goodies->controls4, PANEL_BUTTON,
 			PANEL_LABEL_STRING, "Dismiss", PANEL_NOTIFY_PROC,
@@ -415,14 +428,15 @@ Notify_client	c;
 int		w;
 {
 	window1_objects	*ip = Workman_window1;
-	static int	old_cdmode, new_image = 0;
+	static int	new_image = 0, initted_volume = 0;
+	static enum	cd_modes old_cdmode;
 	Xv_opaque	old_image;
 
 	if (xv_get(ip->mode, PANEL_VALUE) != 5 || ! dont_retry)
 		switch (cd_status()) {
 		case 0:		/* No CD in drive */
-			cur_cdmode = 5;
-			if (old_cdmode != 5)
+			cur_cdmode = EJECTED;
+			if (old_cdmode != EJECTED)
 			{
 				if (!xv_get(ip->tracks, PANEL_INACTIVE))
 				{
@@ -437,7 +451,7 @@ int		w;
 			if (update_everything(ip))
 				return (handle_timer(c, w));
 
-			if (cur_cdmode != 4 && cur_cdmode != 3 ||
+			if (cur_cdmode != STOPPED && cur_cdmode != PAUSED ||
 						old_cdmode != cur_cdmode)
 				show_stats(ip);
 
@@ -473,7 +487,7 @@ int		w;
 update_everything(ip)
 	window1_objects *ip;
 {
-	if (cur_cdmode == 0)		/* Done with track... */
+	if (cur_cdmode == TRACK_DONE)		/* Done with track... */
 	{
 donewithcd:
 		if (xv_get(Workman_goodies->abrepeat, PANEL_VALUE))
@@ -493,7 +507,7 @@ donewithcd:
 		play_next_entry();
 		if (cd_status() != 1)
 			return (1);
-		if (cur_cdmode == 4)	/* Done with CD */
+		if (cur_cdmode == STOPPED)	/* Done with CD */
 		{
 			xv_set(Workman_goodies->abrepeat,
 				PANEL_VALUE, FALSE, NULL);
@@ -543,7 +557,7 @@ donewithcd:
 		cur_track = cur_firsttrack;
 
 	/* The slider has been moved... */
-	if (time_wanted > -1 && cur_cdmode == 1)
+	if (time_wanted > -1 && cur_cdmode == PLAYING)
 	{
 		play_from_pos(time_wanted);
 		time_wanted = -2;
@@ -562,6 +576,13 @@ donewithcd:
 	return (0);
 }
 
+Defaults_pairs autoplay_modes[] = {
+	"never",	0,
+	"normal",	1,
+	"always",	2,
+	NULL,		1
+};
+
 /*
  * A new CD has just been inserted, or possibly we're just starting up and
  * there was already a CD in the drive.  Make all the necessary preparations.
@@ -572,6 +593,7 @@ new_cd_inserted(ip)
 	window1_objects	*ip;
 {
 	static int	initted_volume = 0;
+	static int	autoplay = -1;
 	int		volume, max;
 
 	if (! initted_volume)
@@ -615,7 +637,12 @@ new_cd_inserted(ip)
 	show_stats(ip);
 	cd_status();
 
-	if ((cur_playnew && !found_in_rc) || get_autoplay() || cur_cdmode == 1)
+	/* See if the user has specified an insertion policy in his resources.*/
+	autoplay = defaults_get_enum("workman.autoPlay",
+				"Workman.AutoPlay", autoplay_modes);
+
+	if (autoplay == 2 || (autoplay == 1 && ((cur_playnew && !found_in_rc) ||
+				get_autoplay())) || cur_cdmode == PLAYING)
 		make_initial_playlist(get_playmode());
 }
 
@@ -629,7 +656,7 @@ void
 make_initial_playlist(playmode)
 	int	playmode;
 {
-	if (cur_cdmode == 1)
+	if (cur_cdmode == PLAYING)
 	{
 		if (playmode == 1)
 		{
@@ -679,9 +706,9 @@ change_mode(item, value, event)
 	window1_objects *ip = Workman_window1;
 	int	track, playmode;
 
-	if (cur_cdmode == 5 && dont_retry)
+	if (cur_cdmode == EJECTED && dont_retry)
 		handle_timer(NULL, NULL);
-	if (cur_cdmode == 5 || (cur_track == -1 && (value == 0 || value == 2)))
+	if (cur_cdmode == EJECTED || (cur_track == -1 && (value == 0 || value == 2)))
 	{
 		xv_set(ip->mode, PANEL_VALUE, cur_cdmode, NULL);
 		return;
@@ -689,7 +716,7 @@ change_mode(item, value, event)
 
 	switch (value) {
 	case 0:		/* back a track */
-		if (cur_cdmode == 1)
+		if (cur_cdmode == PLAYING)
 		{
 			play_prev_track();
 			cd_status();
@@ -704,7 +731,7 @@ change_mode(item, value, event)
 			cur_pos_abs = cur_frame / 75;
 		}
 
-		if (cur_cdmode == 1 || cur_cdmode == 4)
+		if (cur_cdmode == PLAYING || cur_cdmode == STOPPED)
 			xv_set(ip->mode, PANEL_VALUE, cur_cdmode, NULL);
 
 		if (xv_get(Workman_goodies->abrepeat, PANEL_VALUE))
@@ -717,13 +744,13 @@ change_mode(item, value, event)
 		break;
 
 	case 1:		/* play */
-		if (cur_cdmode == 3)
+		if (cur_cdmode == PAUSED)
 		{
 			pause_cd();
 			show_stats(ip);
 			break;
 		}
-		if (cur_cdmode == 4)
+		if (cur_cdmode == STOPPED)
 		{
 			/* XXX should call make_initial_playlist() */
 			track = xv_get(ip->tracks, PANEL_VALUE) + 1;
@@ -737,13 +764,13 @@ change_mode(item, value, event)
 				{
 					pl_find_track(track);
 					cur_track = track;
-					cur_cdmode = 1;
+					cur_cdmode = PLAYING;
 					play_from_pos(0);
 					displayed_track = -1;
 				}
 			}
 		}
-		if (cur_cdmode != 1)
+		if (cur_cdmode != PLAYING)
 			play_next_entry();
 		cd_status();
 
@@ -756,10 +783,10 @@ change_mode(item, value, event)
 		break;
 
 	case 2:		/* forward a track */
-		if (cur_cdmode == 1)
+		if (cur_cdmode == PLAYING)
 		{
 			play_next_track();
-			if (cur_cdmode == 4)
+			if (cur_cdmode == STOPPED)
 				goto stopped;
 			cd_status();
 		}
@@ -773,7 +800,7 @@ change_mode(item, value, event)
 			cur_pos_abs = cur_frame / 75;
 		}
 
-		if (cur_cdmode == 1 || cur_cdmode == 4)
+		if (cur_cdmode == PLAYING || cur_cdmode == STOPPED)
 			xv_set(ip->mode, PANEL_VALUE, cur_cdmode, NULL);
 
 		if (xv_get(Workman_goodies->abrepeat, PANEL_VALUE))
@@ -922,11 +949,11 @@ change_track(item, value, event)
 {
 	window1_objects *ip = Workman_window1;
 	
-	if (cur_cdlen > 0 && cur_cdmode != 5)
+	if (cur_cdlen > 0 && cur_cdmode != EJECTED)
 	{
 		if (value == -1)
 		{
-			if (cur_cdmode == 1 || cur_cdmode == 3)
+			if (cur_cdmode == PLAYING || cur_cdmode == PAUSED)
 				xv_set(item, PANEL_VALUE, cur_track - 1, NULL);
 			else
 			{
@@ -940,7 +967,7 @@ change_track(item, value, event)
 		else
 			cur_track = value + 1;
 
-		if (cur_cdmode == 1)
+		if (cur_cdmode == PLAYING)
 		{
 			pl_find_track(cur_track);
 			play_from_pos(0);
@@ -966,7 +993,7 @@ change_pos(item, value, event)
 	
 	time_wanted = value;
 
-	if (cur_cdmode == 4 && cur_track > 0)
+	if (cur_cdmode == STOPPED && cur_track > 0)
 	{
 		if (! xv_get(Workman_goodies->timemode_track, PANEL_VALUE))
 			sprintf(time, "%02d:%02d", value / 60, value % 60);
@@ -1735,11 +1762,11 @@ index_scan(item, event)
 {
 	int	track, index;
 
-	if (cur_cdmode != 4)
+	if (cur_cdmode != STOPPED)
 	{
-		change_mode(NULL, 4, NULL);
-		cur_cdmode = 4;
-		xv_set(Workman_window1->mode, PANEL_VALUE, 4, NULL);
+		change_mode(NULL, STOPPED, NULL);
+		cur_cdmode = STOPPED;
+		xv_set(Workman_window1->mode, PANEL_VALUE, STOPPED, NULL);
 	}
 
 	for (track = 1; track <= cd->ntracks; track++)
@@ -1765,7 +1792,7 @@ byebye(c, s)
 	Notify_client	c;
 	Destroy_status	s;
 {
-	if (s == DESTROY_CHECKING && cur_cdmode != 5)
+	if (s == DESTROY_CHECKING && cur_cdmode != EJECTED)
 	{
 		keep_settings(Workman_window1);
 		if (info_modified)
@@ -1815,11 +1842,11 @@ sigusr1(client, sig, when)
 	int			sig;
 	Notify_signal_mode	when;
 {
-	if (cur_cdmode == 1)
+	if (cur_cdmode == PLAYING)
 	{
-		change_mode(NULL, 3, NULL);
-		cur_cdmode = 3;
-		xv_set(Workman_window1->mode, PANEL_VALUE, 3, NULL);
+		change_mode(NULL, PAUSED, NULL);
+		cur_cdmode = PAUSED;
+		xv_set(Workman_window1->mode, PANEL_VALUE, PAUSED, NULL);
 	}
 
 	return (NOTIFY_DONE);
@@ -1831,11 +1858,11 @@ sigusr2(client, sig, when)
 	int			sig;
 	Notify_signal_mode	when;
 {
-	if (cur_cdmode == 4 || cur_cdmode == 3)
+	if (cur_cdmode == STOPPED || cur_cdmode == PAUSED)
 	{
-		change_mode(NULL, 1, NULL);
-		cur_cdmode = 1;
-		xv_set(Workman_window1->mode, PANEL_VALUE, 1, NULL);
+		change_mode(NULL, PLAYING, NULL);
+		cur_cdmode = PLAYING;
+		xv_set(Workman_window1->mode, PANEL_VALUE, PLAYING, NULL);
 	}
 
 	return (NOTIFY_DONE);
@@ -1847,11 +1874,11 @@ sigquit(client, sig, when)
 	int			sig;
 	Notify_signal_mode	when;
 {
-	if (cur_cdmode == 1 || cur_cdmode == 3)
+	if (cur_cdmode == PLAYING || cur_cdmode == PAUSED)
 	{
-		change_mode(NULL, 4, NULL);
-		cur_cdmode = 4;
-		xv_set(Workman_window1->mode, PANEL_VALUE, 4, NULL);
+		change_mode(NULL, STOPPED, NULL);
+		cur_cdmode = STOPPED;
+		xv_set(Workman_window1->mode, PANEL_VALUE, STOPPED, NULL);
 	}
 
 	return (NOTIFY_DONE);
@@ -1863,7 +1890,7 @@ sigttin(client, sig, when)
 	int			sig;
 	Notify_signal_mode	when;
 {
-	change_mode(NULL, 0, NULL);
+	change_mode(NULL, BACK, NULL);
 
 	return (NOTIFY_DONE);
 }
@@ -1874,7 +1901,7 @@ sigttou(client, sig, when)
 	int			sig;
 	Notify_signal_mode	when;
 {
-	change_mode(NULL, 2, NULL);
+	change_mode(NULL, FORWARD, NULL);
 
 	return (NOTIFY_DONE);
 }
@@ -1885,8 +1912,8 @@ sigemt(client, sig, when)
 	int			sig;
 	Notify_signal_mode	when;
 {
-	change_mode(NULL, 5, NULL);
-	if (cur_cdmode == 5)
+	change_mode(NULL, EJECTED, NULL);
+	if (cur_cdmode == EJECTED)
 		xv_set(Workman_window1->mode, PANEL_VALUE, 5, NULL);
 
 	return (NOTIFY_DONE);
